import ctypes
import os
from PIL import Image
from glob import glob

_lib = ctypes.CDLL(glob('%s/libimproc-opencv*.so' % os.path.dirname(__file__))[0])

# void
# ImProcOcvContrastImproveRgba(int aSourceWidth, int aSourceHeight, int aSourceStride,
#       const void *aSourceData, void *aTargetData)
_proc_rgba = _lib.ImProcOcvContrastImproveRgba
_proc_rgba.argtypes = (ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_void_p, ctypes.c_void_p)


class Improc:
    def __init__(self):
        self._source_wh = (65535, 65535)
        self._target_buffer = None
        self._source_buffer = None

    def improve_contrast_rgba(self, source_im):
        """
        :type source_im: Image.Image
        :rtype: Image.Image
        """
        source_wh = source_im.width, source_im.height
        if self._source_wh != source_wh:
            self._target_buffer = ctypes.create_string_buffer(4 * source_im.width * source_im.height)
            self._source_buffer = ctypes.create_string_buffer(4 * source_im.width * source_im.height)

        with Image.frombuffer(
                'RGBA', (source_im.width, source_im.height), self._source_buffer,
                'raw', 'RGBA', 0, 1) as source_buffer:
            source_buffer.readonly = 0
            source_buffer.paste(source_im, (0, 0))

        _proc_rgba(
            source_im.width, source_im.height, 0,
            ctypes.cast(self._source_buffer, ctypes.c_void_p),
            ctypes.cast(self._target_buffer, ctypes.c_void_p))

        pil_image = Image.frombuffer(
            'RGBA', (source_im.width, source_im.height),
            self._target_buffer, 'raw', 'RGBA', 0, 1)

        return pil_image
